# -*- coding: utf-8 -*-
""" Viewing components for WAeUP objects.
"""
import copy
import grok
import os
import sys
from hurry import yui
from hurry.query import Eq
from hurry.query.query import Query, Text
from hurry.workflow.interfaces import NoTransitionAvailableError
from waeup.browser import (WAeUPPage, WAeUPEditFormPage, WAeUPAddFormPage,
                           WAeUPDisplayFormPage, NullValidator)
from waeup.catalog import search, search_context
from waeup.datacenter import Import
from waeup.interfaces import (IUniversity, IWAeUPObject, IUserContainer,
                              IUserAccount, IFacultyContainer, IDataCenter,
                              IFaculty, IDepartment, ICourse, ICertificate,
                              ICertificateCourse, IWAeUPXMLImporter,
                              IWAeUPXMLExporter)
from waeup.widgets.interfaces import ITableProvider
from waeup.utils.helpers import getName
from zope import schema
from zope.app.security.interfaces import (IAuthentication,
                                          IUnauthenticatedPrincipal,
                                          ILogout)
from zope.component import getUtility, createObject
from zope.component.interfaces import Invalid
from zope.exceptions import DuplicationError

grok.context(IWAeUPObject)
grok.templatedir('templates')

#
# Login/logout pages...
#

class LoginPage(WAeUPPage):
    """A login page, available for all objects.
    """
    grok.name('login')
    grok.context(IWAeUPObject)

    title = u'Login'
    camefrom = None

    def update(self, SUBMIT=None, camefrom=None):
        self.camefrom = camefrom
        if SUBMIT is not None:
            if self.request.principal.id != 'zope.anybody':
                self.flash('You logged in.')
                if not self.camefrom:
                    # User might have entered the URL directly. Let's beam
                    # him back to our context.
                    self.redirect(self.url(self.context))
                    return
                self.redirect(self.camefrom)
                return
            self.flash('You entered wrong credentials!')


class LoginStaffPage(LoginPage):
    """A login page for staff members.
    """
    grok.name('loginstaff')
    grok.template('loginstaffpage')
    title = u'Staff Login'


class LoginStudentPage(LoginPage):
    """A login page for students.
    """
    grok.name('loginstudent')
    grok.template('loginstudentpage')
    title = u'Student Login'

class LogoutPage(WAeUPPage):
    """A logout page. Calling this page will log the current user out.
    """
    grok.context(IWAeUPObject)
    grok.name('logout')
    
    def update(self):
        if not IUnauthenticatedPrincipal.providedBy(self.request.principal):
            auth = getUtility(IAuthentication)
            ILogout(auth).logout(self.request)
            self.flash("You have been logged out. Thanks for using WAeUP SRP!")
        self.redirect(self.url(self.context))


#
# University related pages...
#

class UniversityPage(WAeUPPage):
    """ The main university page.
    """
    grok.name('index')
    grok.context(IUniversity)
    title = u'Welcome to SRP'
    pnav = 0

class Manage(WAeUPEditFormPage):
    """Manage the basic properties of a `University` instance.
    """
    form_fields = grok.AutoFields(IUniversity)
    grok.context(IUniversity)
    grok.require('waeup.manageUniversity')
    title = u'Edit settings'
    pnav = 0

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('Settings have been saved.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        self.flash('Settings have been saved.')
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.flash('Action cancelled.')
        self.redirect(self.url(self.context))
        return

#
# User container pages...
#
    
class UserContainerPage(WAeUPPage):
    """Overview page for all local users.
    """
    grok.require('waeup.manageUsers')
    grok.context(IUserContainer)
    grok.name('index')
    title = 'Users'
    
    def update(self, userid=None, adduser=None, edit=None, delete=None):
        if edit is not None and userid is not None:
            self.redirect(self.url(userid))
        if delete is not None and userid is not None:
            self.context.delUser(userid)
            self.flash('User %s successfully deleted.' % userid)

class AddUserFormPage(WAeUPAddFormPage):
    grok.require('waeup.manageUsers')
    grok.context(IUserContainer)
    grok.name('add')
    form_fields = grok.AutoFields(IUserAccount)
    label = 'Add user'
    title = 'Add user'

    @grok.action('Add user')
    def addUser(self, **data):
        name = data['name']
        title = data['title']
        description = data['description']
        password = data['password']
        roles = data['roles']
        try:
            self.context.addUser(name, password, title=title,
                                 description=description, roles=roles)
        except DuplicationError:
            self.status = Invalid('The userid chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

class UserEditFormPage(WAeUPEditFormPage):
    """Edit a user account.
    """
    grok.context(IUserAccount)
    grok.name('index')
    
    form_fields = grok.AutoFields(IUserAccount)
    grok.require('waeup.manageUsers')

    title = "Edit user"

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('User settings have been saved.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.flash('User settings have been saved.')
        self.redirect(self.url(self.context.__parent__))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context.__parent__))
        return

#
# Search pages...
#
class SearchStudentPage(WAeUPPage):
    grok.context(IUniversity)
    grok.name('searchstudent')
    grok.template('searchstudentpage')
    grok.require('waeup.View')
    title = u"Student Search"
    pnav = 2

    def update(self, query=None):
        self.search_result = []
        if query is not None:
            self.search_result = search_context(query)
            print "RESULT: ", list(self.search_result)


class SearchPage(WAeUPPage):
     grok.context(IUniversity)
     grok.name('search')
     grok.template('searchpage')
     grok.require('waeup.View')
     title = u"Site Search"
     pnav = 2

     def update(self, *args, **kw):
          form = self.request.form
          self.hitlist = []
          self.query = ''
          if not 'query' in form:
               return
          query = form['query']
          self.query = query
          self.hitlist = search(query=self.query, view=self)
          return

#
# Datacenter pages...
#

class DatacenterPage(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('index')
    title = u'Data Center'
    pnav = 0

class DatacenterUploadPage(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('upload')
    title = u'Data Center Upload'
    pnav = 0
    
    def update(self, uploadfile=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not uploadfile:
            return
        try:
            filename = uploadfile.filename
            target = os.path.join(self.context.storage, filename)
            open(target, 'wb').write(uploadfile.read())
        except IOError:
            self.flash('Error while uploading file. Please retry.')
            self.flash('I/O error: %s' % sys.exc_info()[1])
            return
        self.redirect(self.url(self.context))


class DataCenterImportCSVPage(WAeUPPage):
    """A view for importing things.

    """
    grok.require('waeup.manageUniversity')
    grok.name('importcsv')
    grok.template('datacenterimportcsvpage')
    grok.context(IDataCenter)
    title = u'Import CSV'
    pnav = 0

    def getImports(self):
        result = []
        imports = self.context.getPossibleImports()
        result = [Import(x, y) for x, y in imports]
        return result

    def update(self, csvfile=None, clear=None, overwrite=None,
               receiverid=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not csvfile:
            return
        if not SUBMIT:
            return
        try:
            self.context.doImport(csvfile, receiverid, clear=clear)
        except ValueError:
            self.flash('Could not import: %s' % os.path.basename(csvfile))
            return
        except:
            self.flash('Import failed: %s %s' % (sys.exc_info()[0],
                                                 sys.exc_info()[1]))
            return
        self.flash('Successfully imported: %s' % os.path.basename(csvfile))
        self.redirect(self.url(self.context))
        return

class DatacenterSettings(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('manage')
    grok.template('datacentermanagepage')
    grok.require('waeup.manageUniversity')
    title = u'Data Center Settings'
    pnav = 0
    #grok.template('master')

    def update(self, newpath=None, move=False, overwrite=False,
               save=None, cancel=None):
        if move:
            move = True
        if overwrite:
            overwrite = True
        if newpath is None:
            return
        if cancel is not None:
            self.redirect(self.url(self.context))
            return
        try:
            not_copied = self.context.setStoragePath(newpath, move=move)
            for name in not_copied:
                self.flash('File already existed (not copied): %s' % name)
        except ValueError:
            self.flash('Given storage path cannot be used.')
            self.flash('Error: %s' %sys.exc_info()[1])
            return
        if newpath:
            self.flash('New storage path succefully set.')
            self.redirect(self.url(self.context))
        return

class ExportXMLPage(grok.View):
    """Deliver an XML representation of the context.
    """
    grok.name('export.xml')
    grok.require('waeup.manageUniversity')
    
    def render(self):
        exporter = IWAeUPXMLExporter(self.context)
        xml = exporter.export().read()
        self.response.setHeader(
            'Content-Type', 'text/xml; charset=UTF-8')
        return xml

class ImportXMLPage(WAeUPPage):
    """Replace the context object by an object created from an XML
       representation.

       XXX: This does not work for ISite objects, i.e. for instance
            for complete University objects.
    """
    grok.name('importxml')
    grok.require('waeup.manageUniversity')

    def update(self, xmlfile=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not xmlfile:
            return
        importer = IWAeUPXMLImporter(self.context)
        obj = importer.doImport(xmlfile)
        if type(obj) != type(self.context):
            return
        parent = self.context.__parent__
        name = self.context.__name__
        self.context = obj
        if hasattr(parent, name):
            setattr(parent, name, obj)
        else:
            del parent[name]
            parent[name] = obj
            pass
        return



#
# FacultyContainer pages...
#

class FacultyContainerPage(WAeUPPage):
    """ Index page for faculty containers.
    """
    grok.context(IFacultyContainer)
    grok.require('waeup.View')
    grok.name('index')
    title = 'Faculties'
    pnav = 1

    def update(self):
        self.table = ITableProvider(self.context).getTables(self)[0]
        self.table.need()
    
    def getFaculties(self):
        """Convinience method to create a sorted list of faculties.

        It provides a list of dicts with entries for all data needed by
        usual list templates.
        """
        result = []
        for key, val in self.context.items():
            result.append(dict(id=key, name=val.title))
        return result

class AddFacultyFormPage(WAeUPAddFormPage):
    """ Page form to add a new faculty to a faculty container.
    """
    grok.context(IFacultyContainer)
    grok.require('waeup.manageUniversity')
    grok.name('add')
    form_fields = grok.AutoFields(IFaculty)
    label = 'Add a faculty'
    title = 'Add a faculty'
    pnav = 1

    @grok.action('Add faculty')
    def addFaculty(self, **data):
        faculty = createObject(u'waeup.Faculty')
        self.applyData(faculty, **data)
        try:
            self.context.addFaculty(faculty)
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

class ManageFacultyContainerPage(WAeUPPage):
    """ Manage Page 
    """
    grok.context(IFacultyContainer)
    grok.require('waeup.manageUniversity')
    grok.template('facultycontainermanagepage')
    grok.name('manage')
    title = "Manage Faculties"
    pnav = 1

    def update(self):
        form = self.request.form
        if 'CANCEL' in form.keys():
            self.redirect(self.url(self.context))
        if not 'DELETE' in form.keys():
            return
        fac_id = form['fac_id']
        if not isinstance(fac_id, list):
            fac_id = [fac_id]
        deleted = []
        for id in fac_id:
            try:
                del self.context[id]
                deleted.append(id)
            except:
                self.flash('Could not delete %s: %s: %s' % (
                        id, sys.exc_info()[0], sys.exc_info()[1]))
        if len(deleted):
            self.flash('Successfully deleted: %s' % ', '.join(deleted))
        # We have to redirect to let flash messages appear immediately...
        self.redirect(self.url())
        return

#
# Faculty pages
#
class FacultyPage(WAeUPPage):
    """Index page of faculties.
    """
    grok.context(IFaculty)
    grok.require('waeup.View')
    grok.name('index')
    pnav = 1
    @property
    def title(self):
        return "%s (%s)" % (getName(self.context), self.context.code)

    def update(self):
        self.table = ITableProvider(self.context).getTables(view=self)[0]
        self.table.need()

class AddDepartmentFormPage(WAeUPAddFormPage):
    """Add a department to a faculty.
    """
    grok.context(IFaculty)
    grok.name('add')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(IDepartment)
    title = 'Add a department'
    pnav = 1
    
    @grok.action('Add department')
    def addDepartment(self, **data):
        department = createObject(u'waeup.Department')
        self.applyData(department, **data)
        try:
            self.context.addDepartment(department)
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

class ManageFacultyFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `Faculty` instance.
    """
    grok.context(IFaculty)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(IFaculty)
    title = 'Edit faculty'
    pnav = 1

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

#
# Department pages
#
class DepartmentPage(WAeUPPage):
    """Department index page.
    """
    grok.context(IDepartment)
    grok.require('waeup.View')
    grok.name('index')
    pnav = 1

    @property
    def title(self):
        return "%s (%s)" % (getName(self.context), self.context.code)
    
    def update(self):
        yui.tabview.need()

        if 'delcourse' in self.request.form:
            code = self.request.form['code']
            del self.context.courses[code]
        if 'delcert' in self.request.form:
            code = self.request.form['code']
            del self.context.certificates[code]
        self.tables = ITableProvider(self.context).getTables(view=self)
        self.coursetable = self.tables[0]
        self.certificatetable = self.tables[1]
        self.coursetable.need()

class DepartmentManageFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `Department` instance.
    """
    grok.context(IDepartment)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    title = u'Edit department settings'
    pnav = 1

    form_fields = grok.AutoFields(IDepartment)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class DepartmentAddCourseFormPage(WAeUPAddFormPage):
    """Add-form to add course to a department.
    """
    grok.context(IDepartment)
    grok.name('addcourse')
    grok.require('waeup.manageUniversity')
    title = u'Add course'
    pnav = 1
    
    # We need a deepcopy here, because otherwise also all other
    # ICourse forms would be affected.
    form_fields = copy.deepcopy(grok.AutoFields(ICourse))
    form_fields.get('code').field.readonly = False

    @grok.action('Add course')
    def addCourse(self, **data):
        course = createObject(u'waeup.Course')
        self.applyData(course, **data)
        try:
            self.context.courses.addCourse(course)
        except DuplicationError:
            self.status = Invalid('The code chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class DepartmentAddCertificateFormPage(WAeUPAddFormPage):
    """Add-form to add certificate to a department.
    """
    grok.context(IDepartment)
    grok.name('addcertificate')
    grok.require('waeup.manageUniversity')
    title = u'Add certificate'
    pnav = 1
    form_fields = grok.AutoFields(ICertificate)
    
    @grok.action('Add certificate')
    def addCertificate(self, **data):
        certificate = createObject(u'waeup.Certificate')
        try:
            self.applyData(certificate, **data)
        except NoTransitionAvailableError:
            self.status = Invalid('Review state cannot be changed to '
                                  'requested state: no such transition '
                                  'available in workflow.')
            return
        try:
            self.context.certificates.addCertificate(certificate)
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self): #, **data):
        self.redirect(self.url(self.context))
        return

#
# Courses pages
#
class CoursePage(WAeUPDisplayFormPage):
    """Course index page.
    """
    grok.context(ICourse)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1
    form_fields = grok.AutoFields(ICourse)

    @property
    def title(self):
        return 'Course: %s (%s)' % (self.context.title, self.context.code)

class CourseManageFormPage(WAeUPEditFormPage):
    """Edit form page for courses.
    """
    grok.context(ICourse)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    title = u'Edit course settings'
    pnav = 1

    form_fields = grok.AutoFields(ICourse)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

#
# Certificate pages
#
class CertificatePage(WAeUPPage):
    """Index page for certificates.
    """
    grok.context(ICertificate)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1

    @property
    def title(self):
        return "%s (%s)" % (self.context.title, self.context.code)

    def update(self):
        if 'delcourse' in self.request.form:
            code = self.request.form['code']
            level = self.request.form['level']
            self.context.delCourseRef(code, level=level)
        return
        
    def getCourseList(self):
        def cmpCourses(a, b):
            if a['level'] < b['level']:
                return -1
            if a['level'] > b['level']:
                return 1
            if a['mandatory'] != b['mandatory']:
                if a['mandatory'] == False and b['mandatory'] == True:
                    return -1
                return 1
            if a['code'] < b['code']:
                return -1
            if a['code'] > b['code']:
                return 1
            return 0

        entries = []
        levels = dict()
        for key, course in self.context.items():
            certcourse = self.context[key]
            course = certcourse.course
            level = certcourse.level
            mandatory = certcourse.core_or_elective
            title = course.title
            code = course.code
            course_url = self.url(self.context[key])
            semester = course.semester

            new_entry = dict(
                level=level, mandatory=mandatory, url=course_url,
                title=title, code=code, semester=semester)
            if str(level) not in levels.keys():
                levels[str(level)] = dict()
            if str(semester) not in levels[str(level)].keys():
                levels[str(level)][str(semester)] = []
            levels[str(level)][str(semester)].append(new_entry)
        return levels

class CertificateManageForm(WAeUPEditFormPage):
    """Manage the basic properties of a `Certificate` instance.
    """
    grok.context(ICertificate)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    # We omit 'code' field from original schema and set prepend a new
    # one with 'readonly' set to True.
    form_fields = grok.Fields(
        code = schema.TextLine(
            title = u'Code', readonly=True
            )
        ) + grok.AutoFields(ICertificate).omit('code')
    pnav = 1

    @property
    def title(self):
        # Set certificate name as form label
        return self.form_fields['title'].field.get(self.context)
    
    @grok.action('Save')
    def save(self, **data):
        try:
            self.applyData(self.context, **data)
        except NoTransitionAvailableError:
            self.status = Invalid('Review state cannot be changed to '
                                  'requested state: no such transition '
                                  'available in workflow.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        try:
            self.applyData(self.context, **data)
        except NoTransitionAvailableError:
            self.status = Invalid('Review state cannot be changed to '
                                  'requested state: no such transition '
                                  'available in workflow.')
            return
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

#
# Certificate course pages...
#
class CertificateCoursePage(WAeUPDisplayFormPage):
    """Course index page.
    """
    grok.context(ICertificateCourse)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1
    form_fields = grok.AutoFields(ICertificateCourse)

    @property
    def title(self):
        return self.context.course.title

class CertificateCourseFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `CertificateCourse` instance.
    """
    grok.context(ICertificateCourse)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(ICertificateCourse)
    pnav = 1
    
    @property
    def title(self):
        return 'Edit certificate course %s' % self.context.course.code
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        parent = self.context.__parent__
        if self.context.level == data['level']:
            self.applyData(self.context, **data)
        else:
            # The level changed. We have to create a new certcourse as
            # the key of the entry will change as well...
            old_level = self.context.level
            parent.addCourseRef(**data)
            parent.delCourseRef(data['course'].code, level=old_level)
        self.redirect(self.url(parent))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class CertificateCourseAddFormPage(WAeUPAddFormPage):
    """Add-page to add a course ref to a certificate
    """
    grok.context(ICertificate)
    grok.name('addcertificatecourse')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(ICertificateCourse)
    pnav = 1
    title = 'Add a course'

    @grok.action('Add course')
    def addCertcourse(self, **data):
        try:
            self.context.addCourseRef(**data)
        except DuplicationError:
            self.status = Invalid('The chosen course is already part of'
                                  'this certificate')
            return
        self.redirect(self.url(self.context))

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return
